//  BITCOIN DOLLAR TICKER    John Dingley 04.10.19

/*********************************************************************************************************************************
 *  I wrote this based upon an example of getting ticker numerical info from coinmarketcap.com                                   *
 *  originally written by Brian Lough.                                                                                           *                             
 *  https://platformio.org/lib/show/1692/CoinMarketCapApi                                                                        *                                                             
 *                                                                                                                               *
 *  Also see his Instructable here   https://www.instructables.com/id/Simple-10-Crypto-Currency-Display/                         *
 *  I then wrote the graphing part myself along with the code to turn the rising or falling LEDs on or off.                      *
 *  Also there is an attempt to add indicators on screen showng when the trend may be reversing direction in the very short term *
 *********************************************************************************************************************************/


 /****************************************************************************************************************************************************
  * The inflection point markers take time to stabilise so ignore the first few readings.                                                            *
  * The y axis auto adjusts. If there is a sudden pump or dump in price such that the curve is in danger of going off top or bottom of the graph,    *
  * the graph will restart from left with re-adjusted y axis                                                                                         *
  * The reason I have done this is so that even when price is quite stable, small changes in price are always visible as a change in the curve shape *
  * The last 2 digits of the previous few prices are shown on the right, most recent at the top of the column of 4 values                            *
  * The percentage change x10 since the previous reading approx 2 min earlier is also displayed top right with the D in front of it                  *
  * so you can spot sudden large changes                                                                                                             *
  * The value top right with Av in front of it is the average of the last few percentage change (x10) values.                                        *
  * If the D value differs from the Av value by more than a trigger value of 0.8 or -0.8, then this means a fast change has suddenly happened        *
  * This will cause screen to flash, LED's to flash and an ALERT signal to appear on screen. This tells you to pay attention and trade or not        *
  * as you want to.                                                                                                                                  *
  ****************************************************************************************************************************************************/

//                    Wiring
// Adafruit OLED pin         ESP pin

// DATA                        4
// CLK                         5
// RST                         2
// Vin                         3.3V supply from ESP
// GND                         GND

// Plus 2 LEDs via 330 Ohm resistors as pin mAmps is only 12 i.e. less than than on a regular Arduino



#include <CoinMarketCapApi.h>



#include <SPI.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define OLED_RESET 2   //rst wire on SSD1306 OLED display is connected to digital 2 on my huzzahESP


#include <ESP8266WiFi.h>
#include <WiFiClientSecure.h>

//------- Install From Library Manager -------
#include <ArduinoJson.h>


//********************************************* USER CONFIGURABLE VARIABLES ***************************************
//The top and bottom of Y axis are automatically configured, each time the graph renews from the left side,  to be the highest previous value plus the plusminus value and lowest previous value minus the plusminus value
//Setting plusminus to about 20 or 30 therefore seems reasonable. Increase it perhaps in times of very high short term volatility.
float plusminus = 30;
float suddenchangetrigger = 0.8;  // If more than this it goes berserk asa  warning something sudden has happened
//This is so the swings in price become more obvious even if they are small
//*****************************************************************************************************************


float upperaxislimit;
float loweraxislimit;

float priceVSrollingmean;

float lowprice;
float highprice;

int priceondisplay;

float rollingmean;

float lastupdated;
float previouslastupdated;


float divideby;

float mdesigned;
float shorttermvolatility;


//------- Replace the following! ------
char ssid[] = "XXXXXXXX";       // your network SSID (name)
char password[] = "XXXXXXX";  // your network key


WiFiClientSecure client;
CoinMarketCapApi api(client);
 


// CoinMarketCap's limit is "no more than 10 per minute"
// Make sure to factor in if you are requesting more than one coin.

unsigned long api_mtbs = 30000; //mean time between api requests i.e. 30 seconds. Maximum allowed by coinmarketcap is 10 requests per minute. 
//However it only seems to update about every 2 minutes so sending in a request every 30 seconds seems to be frequent enough.

unsigned long api_due_time = 0;




#define LED_PIN_UP 14
#define LED_PIN_DOWN 12

float previousValue;
float threshold = 0.05;
float price;
//float SGprice;




Adafruit_SSD1306 display(OLED_RESET);

#define NUMFLAKES 10
#define XPOS 0
#define YPOS 1
#define DELTAY 2



// Adafruit logo. Must be displayed at start like this if making software availbel for others, which I am.
#define LOGO16_GLCD_HEIGHT 16 
#define LOGO16_GLCD_WIDTH  16 
static const unsigned char PROGMEM logo16_glcd_bmp[] =
{ B00000000, B11000000,
  B00000001, B11000000,
  B00000001, B11000000,
  B00000011, B11100000,
  B11110011, B11100000,
  B11111110, B11111000,
  B01111110, B11111111,
  B00110011, B10011111,
  B00011111, B11111100,
  B00001101, B01110000,
  B00011011, B10100000,
  B00111111, B11100000,
  B00111111, B11110000,
  B01111100, B11110000,
  B01110000, B01110000,
  B00000000, B00110000 };



int y;

int x;

int16_t v;
int16_t vold;

int16_t t = 4;
int16_t told;

float m;

float cycles;

float p0, p1, p2, p3, p4, p5, p6;  //Variables to show previous prices


//float d0, d1, d2, d3, d4, d5, d6;    //variables to look at the underlying trend, SG filtered so as to compare what is happening versus the rolling mean price e.g. falling, rising, high point or low point.

int highflag;
int lowflag;

int abridged;
int num1;
int num2;




void setup() {
  
  delay(1000);

  pinMode(LED_PIN_DOWN, OUTPUT);
  pinMode(LED_PIN_UP, OUTPUT);

  Serial.begin(115200);
  delay(500);
  Serial.println("Starting");
  delay(500);

  // Set WiFi to station mode and disconnect from an AP if it was Previously
  // connected
  WiFi.mode(WIFI_STA);
  WiFi.disconnect();
  delay(100);

  // Attempt to connect to Wifi network:
  Serial.print("Connecting Wifi: ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    Serial.print(".");
    delay(500);
  }
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  IPAddress ip = WiFi.localIP();
  Serial.println(ip);


  // by default, we'll generate the high voltage from the 3.3v line internally! (neat!)
  display.begin(SSD1306_SWITCHCAPVCC, 0x3D);  // initialize with the I2C addr 0x3D (for the 128x64)

  
  // Show image buffer on the display hardware.
  // Since the buffer is intialized with an Adafruit splashscreen
  // internally, this will display the splashscreen.
  display.display();
  delay(200);
  
     display.clearDisplay();
   
                 
                display.setTextSize(2);
                display.setTextColor(WHITE);
                display.setCursor(0,0);
                

               display.println(" WELCOME");
               display.setTextSize(1);
               display.println(" ");
               display.println("BITCOIN TICKER");
               display.println(" ");
               display.println("John Dingley 04/10/19");
               display.display();

               delay(3000);
               display.clearDisplay();

  // Flash LED
  digitalWrite(LED_PIN_DOWN, HIGH);
  delay(300);
  digitalWrite(LED_PIN_DOWN, LOW);
  delay(300);
  digitalWrite(LED_PIN_DOWN, HIGH);
  delay(300);
  digitalWrite(LED_PIN_DOWN, LOW);
  delay(300);
  digitalWrite(LED_PIN_DOWN, HIGH);
  delay(300);
  digitalWrite(LED_PIN_DOWN, LOW);
  delay(300);
  digitalWrite(LED_PIN_DOWN, HIGH);
  delay(300);
  digitalWrite(LED_PIN_DOWN, LOW);
  delay(300);
  
  digitalWrite(LED_PIN_UP, HIGH);
  delay(300);
  digitalWrite(LED_PIN_UP, LOW);
  delay(300);
  digitalWrite(LED_PIN_UP, HIGH);
  delay(300);
  digitalWrite(LED_PIN_UP, LOW);
  delay(300);
  digitalWrite(LED_PIN_UP, HIGH);
  delay(300);
  digitalWrite(LED_PIN_UP, LOW);
  delay(300);
  digitalWrite(LED_PIN_UP, HIGH);
  delay(300);
  digitalWrite(LED_PIN_UP, LOW);


  // Draw axes
  

  display.drawLine(0, 63, 115, 63, WHITE);  //draw horizontal axis
  display.drawLine(0, 17, 0, 63, WHITE);  //draw vertical axis
  display.display();

  
               
  delay(3000);
} //end of setup












void printTickerData(String ticker) {
  Serial.println("---------------------------------");
  Serial.println("Getting ticker data for " + ticker);


  // Ticker unfortunately is not the symbol for some reason.
  // Go to CoinMarketCap.com and select the coin you would like to check
  // The ticker name makes up the last part of the URL
  // e.g: http://coinmarketcap.com/currencies/bitcoin/ , "bitcoin" is the ticker value

  // Currency is optional, so you can pass only ticker if you want.
  // Check out the currency drop down on CoinMarketCap.com to get available values
 
  //https://www.binance.com/trade.html?symbol=ETH_USD
  
  CMCTickerResponse response = api.GetTickerInfo(ticker, "eur");   // the default one the program came with that we know works OK
 
  //CMCTickerResponse response = api.GetTickerInfo(ticker, "bitcoin");  //NOTE this works just fine too

 
  
  if (response.error == "") {
    Serial.print("ID: ");
    Serial.println(response.id);
    Serial.print("Name: ");
    Serial.println(response.name);
    Serial.print("Symbol: ");
    Serial.println(response.symbol);

    Serial.print("Rank: ");
    Serial.println(response.rank);

    Serial.print("Price in USD: ");
    Serial.println(response.price_usd);
    Serial.print("Price in BTC: ");
    Serial.println(response.price_btc);

    Serial.print("24h Volume USD: ");
    Serial.println(response.volume_usd_24h);
    Serial.print("Market Cap USD: ");
    Serial.println(response.market_cap_usd);

    Serial.print("Available Supply: ");
    Serial.println(response.available_supply);
    Serial.print("Total Supply: ");
    Serial.println(response.total_supply);

    Serial.print("Percent Change 1h: ");
    Serial.println(response.percent_change_1h);
    Serial.print("Percent Change 24h: ");
    Serial.println(response.percent_change_24h);
    Serial.print("Percent Change 7d: ");
    Serial.println(response.percent_change_7d);
    Serial.print("Last Updated: ");
    Serial.println(response.last_updated);

    // These fields will not come back if you do not request a currency
    Serial.print("Price in requested currecy: ");
    Serial.println(response.price_currency);
    Serial.print("24h Volume in requested currency: ");
    Serial.println(response.volume_currency_24h);
    Serial.print("Market Cap in requested currency: ");
    Serial.println(response.market_cap_currency);


  } else {
    Serial.print("Error getting data: ");
    Serial.println(response.error);
  }
  Serial.println("---------------------------------");







    previouslastupdated = lastupdated;
    lastupdated = response.last_updated;
    
    
    
    

  








    

    if(cycles == 0){

      erasepartsofscreen();
      
                price = response.price_usd;
                priceondisplay = (int)(price + 0.5);    // converts it from float to an integer
                
                display.setTextSize(2);
                display.setTextColor(WHITE);
                display.setCursor(4,0);
                //display.println(response.price_usd);
                display.println(priceondisplay);

      
                highprice = price;
                lowprice = price;
                //rollingmean = price;
                upperaxislimit = price + plusminus + 5; //these are used to create the y axis upper and lower limit for the first screen of graph thereafter the vertical self adjusts according the maximum and minimums seen in previous screen of data
                loweraxislimit = price - plusminus - 5;

                p0 = price;  //starting values
                p1 = price;
                p2 = price;
                p3 = price;
                p4 = price;
                p5 = price;
                p6 = price;
                    }


// XXXXXXXXXXXXXXXXX Update graph and update SG filter with the new variable XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
    if(lastupdated == previouslastupdated){
                      Serial.println("Graph NOT updated as new value from web has not been updated yet");
                                           }

    if((cycles > 0) && (lastupdated != previouslastupdated)){        // check not on first cycle of program and also that the data from api has actually been updated. 
      //This does assume api data is updated at regular and similar intervals else x axis of graph would end up incorrect at would have variable time intervals between each x axis data point
      // If data has not been updated from website then the numerical price display is updated anyway but the graph is not added to until a new value comes from web.

    erasepartsofscreen();


    price = response.price_usd;
    priceondisplay = (int)(price + 0.5);  
    display.setTextSize(2);
    display.setTextColor(WHITE);
    display.setCursor(4,0);
    //display.println(response.price_usd);
    display.println(priceondisplay);



   
    p0 = p1;    
    p1 = p2;
    p2 = p3;
    p3 = p4;
    p4 = p5;
    p5 = p6;
    p6 = price;

    display.setTextSize(1);
    display.setTextColor(WHITE);
    
    display.setCursor(109,21);
    num1 = (int)(p5 + 0.5);  //remove all after decimal point and convert to an integer
    num2 = (num1 % 100);
    Serial.print(">>>>>>>>>>>>>>>>>>>>>>>>>>>>> p5short = ");
    Serial.println(num2);
    if(num2 >= 10){
    display.println(num2);
    }
    else{
      display.setCursor(109,21);
      display.println("0");
      display.setCursor(115,21);
      display.println(num2);    
    }

    display.setCursor(109,31);
    num1 = (int)(p4 + 0.5);  //remove all after decimal point and convert to an integer
    num2 = (num1 % 100);
    Serial.print(">>>>>>>>>>>>>>>>>>>>>>>>>>>>> p4short = ");
    Serial.println(num2);
    if(num2 >= 10){
    display.println(num2);
    }
    else{
      display.setCursor(109,31);
      display.println("0");
      display.setCursor(115,31);
      display.println(num2);    
    }


    display.setCursor(109,41);
    num1 = (int)(p3 + 0.5);  //remove all after decimal point and convert to an integer
    num2 = (num1 % 100);
    Serial.print(">>>>>>>>>>>>>>>>>>>>>>>>>>>>> p3short = ");
    Serial.println(num2);
        if(num2 >= 10){
    display.println(num2);
    }
    else{
      display.setCursor(109,41);
      display.println("0");
      display.setCursor(115,41);
      display.println(num2);    
    }
    


    display.setCursor(109,51);
    num1 = (int)(p2 + 0.5);  //remove all after decimal point and convert to an integer
    num2 = (num1 % 100);
    Serial.print(">>>>>>>>>>>>>>>>>>>>>>>>>>>>> p2short = ");
    Serial.println(num2);
        if(num2 >= 10){
    display.println(num2);
    }
    else{
      display.setCursor(109,51);
      display.println("0");
      display.setCursor(115,51);
      display.println(num2);    
    }







  Serial.print(">>>>>>>>>>>>>>>>> NEW PRICE = ");
  Serial.print(price);
  Serial.println(" <<<<<<<<<<<<<<<<<<<<");

  Serial.print(">>>>>>>>>>>>>>>> PREVIOUS PRICE VALUE =");
  Serial.println(previousValue);



  Serial.print(">>>>>>>>>>> p1-p0 = ");
  m = p1 - p0;
  Serial.print(m);
    Serial.print("  p2-p1 = ");
  m = p2 - p1;
  Serial.print(m);
    Serial.print("  p3- p2 = ");
  m = p3 - p2;
  Serial.print(m);
    Serial.print("  p4-p3 = ");
  m = p4 - p3;
  Serial.print(m);
    Serial.print("  p5-p4 = ");
  m = p5 - p4;
  Serial.print(m);
    Serial.print("  p6-p5 = ");
  m = p6 - p5;
  Serial.print(m);
  Serial.println(" <<<<<<<<<<<<<<<");

  m = ((p6-p5)/p6) * 1000;

  
  

 display.setTextSize(1);
    display.setTextColor(WHITE);
    display.setCursor(72,0);
    display.print("Av:");
    display.setCursor(90,0);
    display.println(shorttermvolatility);
    
    display.setCursor(78,9);
    display.print("D:");
    display.setCursor(90,9);
    display.print(m);
    Serial.print(">>>>>>>>>>>>>>>>> Percent change since last reading x 10:");
    Serial.print(m);
    Serial.print("          shorttermvolatility: ");
    Serial.print(shorttermvolatility);

    Serial.println("  <<<<<<<<<<<<<<<<<<");

    
    




    if(highprice < price){
           highprice = price;
           Serial.println("comparing highprice with price");
                                  }
    if(lowprice > price){
           lowprice = price;
           Serial.println("comparing lowprice with price");
                                }



    

// xxxxxxxxxxxxxxxxxxxxxxxxxxxxx Add a Graph xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx


    divideby = (upperaxislimit - loweraxislimit)/ 46;   // as vertical graph axis is 46 pixels in depth

    vold = v;

    v = (int)(46 - ((price + 0.5 - loweraxislimit) / divideby));   //use price NOT the SGprice to create the actual trend graph
    v = v + 16;

    Serial.print("t =");
    Serial.print(t);
    Serial.print("    v=");
    Serial.println(v);
        Serial.print("highprice =");
    Serial.print(highprice);
    Serial.print("    lowprice =");
    Serial.println(lowprice);
            Serial.print("upperaxislimit =");
    Serial.print(upperaxislimit);
    Serial.print("    loweraxislimit =");
    Serial.println(loweraxislimit);
    
    told = t-4;

    if(told >= 4){
    display.drawLine(told, vold, t, v, WHITE);  //next line on graph
                  }

    lookforatop();
    lookforabottom();
    suddenchangedetector();

    Serial.print(">>>>>>>>>>>>>>>>>> highflag = ");
    Serial.print(highflag);
    Serial.print("      lowflag = ");
    Serial.print(lowflag);
    Serial.println(" <<<<<<<<<<<<<<<<<<<<<<<<");


    if (highflag == 1){

      display.drawLine(t-6, 17, t-4, 20, WHITE);    //marker is located 4 pixels to the left of the current data point as the inflexion point is actually at the previous data point, 
      //when compared to current one, which we now find is lower or higher then the preceding trend i.e. is higher or lower then the trend of the previous 4 points
      display.drawLine(t-2, 17, t-4, 20, WHITE);     
    }

    if (lowflag == 1){

      display.drawLine(t-6, 61, t-4, 58, WHITE);  //same reasoning as above
      display.drawLine(t-2, 61, t-4, 58, WHITE);
    }

    
    t = t + 4;  //each data point is 4 pixels to the right along x axis from the previous one on the screen. They are then joined together with a line
    
    if((t > 111) || (price > upperaxislimit) || (price < loweraxislimit)){ 
           t = 4;
           upperaxislimit = price + ((highprice - lowprice)/2) + plusminus;
           loweraxislimit = price - ((highprice - lowprice)/2) - plusminus;
           cycles = 0;

          

           Serial.print(" >>>>>>> ROLLINGMEAN = ");
           Serial.println(rollingmean);

           for (int16_t i=1; i<128; i+=1) {    //Draw a set of vertical black lines over the graph before it resets itself to the start point again
           display.drawLine(i, 16, i, 62, BLACK);  // vertical black line one across to right of the pixel   
                                           }
                 }



    }// end of if cycles greater than 0 and the check has taken place that the data has actually been updated




    
    cycles = cycles + 1;
    
    display.display(); //display the new btc value and the updated graph below

// xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx End of add a Graph xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
    
} // end of print ticker data







void loop() {


  
  unsigned long timeNow = millis();
  if ((timeNow > api_due_time))  {

    
    
    printTickerData("bitcoin");
    //printTickerData("ethereum");
    //printTickerData("dogecoin");
    api_due_time = timeNow + api_mtbs;



    




    
  } // end of if time now etc

    if (previousValue == 0.0) {

  previousValue = price;

}


// Put on red LED if there is a general downward trend
digitalWrite(LED_PIN_DOWN, LOW);

if ((p6 < p5) && (p5 < p4)){
    digitalWrite(LED_PIN_DOWN, HIGH);
    
}


// Put on green LED if there is a general upward trend
digitalWrite(LED_PIN_UP, LOW);
if ((p6 > p5) && (p5 > p4)){
    digitalWrite(LED_PIN_UP, HIGH);
    
}




previousValue = price;









  
}  // end of main void loop







void lookforatop() {
     
     highflag = 0;
     
     if(((p2 + p3) < (p4 + p5)) && (p6 < ((p4 + p5) / 2))){ highflag = 1;}
  
}






void lookforabottom() {
     
     lowflag = 0;
     
     if(((p2 + p3) > (p4 + p5)) && (p6 > ((p4 + p5) / 2))){ lowflag = 1;}
     
}// end of look for a bottom




void suddenchangedetector(){
    // m is percentage change x10 of new price versus previous price
    
    

    shorttermvolatility = (shorttermvolatility * 0.9) + (m * 0.1);         // Gives a rolling average of last few values as an index of very recent fluctualtions, always a positive value
    Serial.print("shorttermvolatility: ");
    Serial.println(shorttermvolatility);
    Serial.print("m: ");
    Serial.println(m);

    if(m < 0){
    if ((m - shorttermvolatility) < (-suddenchangetrigger)){
                                                      // invert the display
                                                      Serial.println("SOMETHING IS HAPPENING");
                                                      Serial.println("PRICE DROPPING");
                                                      display.invertDisplay(true);
                                                      digitalWrite(LED_PIN_DOWN, HIGH);
                                                      digitalWrite(LED_PIN_UP, HIGH);
                                                      delay(1000); 
                                                      display.invertDisplay(false);
                                                      digitalWrite(LED_PIN_DOWN, LOW);
                                                      digitalWrite(LED_PIN_UP, LOW);
                                                      delay(1000);
                                                      display.invertDisplay(true);
                                                      digitalWrite(LED_PIN_DOWN, HIGH);
                                                      digitalWrite(LED_PIN_UP, HIGH);
                                                      delay(1000); 
                                                      display.invertDisplay(false);
                                                      digitalWrite(LED_PIN_DOWN, LOW);
                                                      digitalWrite(LED_PIN_UP, LOW);
                                                      delay(1000);
                                                      display.invertDisplay(true);
                                                      digitalWrite(LED_PIN_DOWN, HIGH);
                                                      digitalWrite(LED_PIN_UP, HIGH);
                                                      delay(1000); 
                                                      display.invertDisplay(false);
                                                      digitalWrite(LED_PIN_DOWN, LOW);
                                                      digitalWrite(LED_PIN_UP, LOW);
                                                      delay(1000);
                                                      display.invertDisplay(true);
                                                      digitalWrite(LED_PIN_DOWN, HIGH);
                                                      digitalWrite(LED_PIN_UP, HIGH);
                                                      delay(1000); 
                                                      display.invertDisplay(false);
                                                      digitalWrite(LED_PIN_DOWN, LOW);
                                                      digitalWrite(LED_PIN_UP, LOW);
                                                      delay(1000);


                                                      display.setTextSize(1);
                                                      display.setTextColor(WHITE); 
                                                      display.setCursor(4,51);
                                                      display.println("ALERT! FALLING!");
                                                                } //end of if mdesigned has moved more than set trigger amount
                                              } // End of if m < 0



 if(m >= 0){
    if ((m - shorttermvolatility) > (suddenchangetrigger)){
                                                      // invert the display
                                                      Serial.println("SOMETHING IS HAPPENING");
                                                      Serial.println("PRICE RISING");
                                                      display.invertDisplay(true);
                                                      digitalWrite(LED_PIN_DOWN, HIGH);
                                                      digitalWrite(LED_PIN_UP, HIGH);
                                                      delay(1000); 
                                                      display.invertDisplay(false);
                                                      digitalWrite(LED_PIN_DOWN, LOW);
                                                      digitalWrite(LED_PIN_UP, LOW);
                                                      delay(1000);
                                                      display.invertDisplay(true);
                                                      digitalWrite(LED_PIN_DOWN, HIGH);
                                                      digitalWrite(LED_PIN_UP, HIGH);
                                                      delay(1000); 
                                                      display.invertDisplay(false);
                                                      digitalWrite(LED_PIN_DOWN, LOW);
                                                      digitalWrite(LED_PIN_UP, LOW);
                                                      delay(1000);
                                                      display.invertDisplay(true);
                                                      digitalWrite(LED_PIN_DOWN, HIGH);
                                                      digitalWrite(LED_PIN_UP, HIGH);
                                                      delay(1000); 
                                                      display.invertDisplay(false);
                                                      digitalWrite(LED_PIN_DOWN, LOW);
                                                      digitalWrite(LED_PIN_UP, LOW);
                                                      delay(1000);
                                                      display.invertDisplay(true);
                                                      digitalWrite(LED_PIN_DOWN, HIGH);
                                                      digitalWrite(LED_PIN_UP, HIGH);
                                                      delay(1000); 
                                                      display.invertDisplay(false);
                                                      digitalWrite(LED_PIN_DOWN, LOW);
                                                      digitalWrite(LED_PIN_UP, LOW);
                                                      delay(1000);


                                                      display.setTextSize(1);
                                                      display.setTextColor(WHITE); 
                                                      display.setCursor(4,51);
                                                      display.println("ALERT! RISING! ");
                                                                } //end of if mdesigned has moved more than set trigger amount
                                              } // End of if m >= 0                                             


    
                           } // end of sudden change detector











void erasepartsofscreen(){

      //erases the old BTC value. I am sure there is a neater way to do it.

    display.drawLine(0, 0, 127, 0, BLACK);  //horizontal
    display.drawLine(0, 1, 127, 1, BLACK);  //horizontal
    display.drawLine(0, 2, 127, 2, BLACK);  //horizontal
    display.drawLine(0, 3, 127, 3, BLACK);  //horizontal
    display.drawLine(0, 4, 127, 4, BLACK);  //horizontal
    display.drawLine(0, 5, 127, 5, BLACK);  //horizontal
    display.drawLine(0, 6, 127, 6, BLACK);  //horizontal
    display.drawLine(0, 7, 127, 7, BLACK);  //horizontal

    display.drawLine(0, 8, 127, 8, BLACK);  //horizontal
    display.drawLine(0, 9, 127, 9, BLACK);  //horizontal
    display.drawLine(0, 10, 127, 10, BLACK);  //horizontal
    display.drawLine(0, 11, 127, 11, BLACK);  //horizontal
    display.drawLine(0, 12, 127, 12, BLACK);  //horizontal
    display.drawLine(0, 13, 127, 13, BLACK);  //horizontal
    display.drawLine(0, 14, 127, 14, BLACK);  //horizontal
    display.drawLine(0, 15, 127, 15, BLACK);  //horizontal

    //overwrite the area on right where the scrolling previous values will go
    display.drawLine(109, 16, 109, 63, BLACK);  //Vertical line
    display.drawLine(110, 16, 110, 63, BLACK);  //Vertical line
    display.drawLine(111, 16, 111, 63, BLACK);  //Vertical line
    display.drawLine(112, 16, 112, 63, BLACK);  //Vertical line
    display.drawLine(113, 16, 113, 63, BLACK);  //Vertical line
    display.drawLine(114, 16, 114, 63, BLACK);  //Vertical line
    display.drawLine(115, 16, 115, 63, BLACK);  //Vertical line
    display.drawLine(116, 16, 116, 63, BLACK);  //Vertical line
    display.drawLine(117, 16, 117, 63, BLACK);  //Vertical line
    display.drawLine(118, 16, 118, 63, BLACK);  //Vertical line
    display.drawLine(119, 16, 119, 63, BLACK);  //Vertical line
    display.drawLine(120, 16, 120, 63, BLACK);  //Vertical line
    display.drawLine(121, 16, 121, 63, BLACK);  //Vertical line

    
    
    
    display.display();   // enact this
}
